/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.model;

import java.util.Date;
import java.util.List;

/**
 * Wraps a <i>CruiseControl</i> continuous integration project.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class CcProject extends Identified {
    /** Name of the project. */
    private String name;
    /** URL of the build result page. */
    private String buildResultUrl;
    /** Label of the latest build. */
    private String buildLabel;
    /** Status of the latest build. */
    private int status;
    /** Timestamp of the latest build. */
    private Date timestamp;

    /** Number of modifications. */
    private int modifications;
    /** Duration of the build as String (e.g. "33 minutes 16 seconds"). */
    private String buildTime;// TODO should not be stored as String, because it's not localizable

    /** Number of all unit test cases. */
    private int tests;
    /** Number of unit test case failures. */
    private int failures;
    /** Number of unit test case errors. */
    private int errors;
    /** All testcases run during this build. */
    private List<CcTest> testCases;

    /** Locator of the tests-per-category chart image. */
    private ZoomableImageLocator testsPerCategoryChartLocator;

    /**
     * By design this should be private to avoid instantiation
     * without discriminator, but Hessian needs default ctor.
     */
    public CcProject() {
        super(null);
    }

    public CcProject(String name, String buildResultUrl, String buildLabel, int status, Date timestamp,
        int modifications, String buildTime,
        int tests, int errors, int failures, List<CcTest> testCases,
        ZoomableImageLocator testsPerCategoryChartLocator) {
        super(name + buildLabel + timestamp.getTime());

        this.name = name;
        this.buildResultUrl = buildResultUrl;
        this.buildLabel = buildLabel;
        this.status = status;
        this.timestamp = timestamp;

        this.modifications = modifications;
        this.buildTime = buildTime;

        this.tests = tests;
        this.errors = errors;
        this.failures = failures;
        this.testCases = testCases;

        this.testsPerCategoryChartLocator = testsPerCategoryChartLocator;
    }

    public String getName() {
        return name;
    }

    public String getBuildResultUrl() {
        return buildResultUrl;
    }

    public String getBuildLabel() {
        return buildLabel;
    }

    public int getStatus() {
        return status;
    }

    public Date getTimestamp() {
        return timestamp;
    }

    public int getModifications() {
        return modifications;
    }

    public String getBuildTime() {
        return buildTime;
    }

    public int getTests() {
        return tests;
    }

    public int getTestsSucceded() {
        return tests - failures - errors;
    }

    public int getFailures() {
        return failures;
    }

    public int getErrors() {
        return errors;
    }

    public List<CcTest> getTestCases() {
        return testCases;
    }

    public ZoomableImageLocator getTestsPerCategoryChartLocator() {
        return testsPerCategoryChartLocator;
    }

    /** Returns its success rate in the [0.0 ... 1.0] range or 0.0 if it had no tests at all. */
    public double getSuccessRate() {
        if(tests == 0)
            return 0.0;

        return ((double)getTestsSucceded())/tests;
    }

    /** Returns its age in milliseconds. */
    public long getLastBuildAge() {
        return new Date().getTime() - timestamp.getTime();
    }

    /** Returns its age in minutes. */
    public long getLastBuildAgeInHours() {
        return getLastBuildAge()/ModelUtils.MILLIS_PER_HOUR;
    }

    public String toString() {
        return String.format("%s, %s, %s, %s, %d, %s, %d/%d/%d, %s", name, buildLabel, status, timestamp, modifications, buildTime, tests, failures, errors, testsPerCategoryChartLocator);
    }
}
