/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.server.jira;

import hu.midori.kosmos.model.JiraProject;
import hu.midori.kosmos.model.ZoomableImageLocator;
import hu.midori.kosmos.protocol.JiraService;
import hu.midori.kosmos.server.util.ChartUtils;
import hu.midori.kosmos.server.util.UrlUtils;

import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.rpc.ServiceException;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import _soapclient.JiraSoapService;
import _soapclient.JiraSoapServiceService;
import _soapclient.JiraSoapServiceServiceLocator;

import com.atlassian.jira.rpc.soap.beans.RemoteFilter;
import com.atlassian.jira.rpc.soap.beans.RemoteIssue;
import com.atlassian.jira.rpc.soap.beans.RemoteProject;

/**
 * Implementation of the <i>JIRA</i> service.
 * <p>
 * Uses the JIRA SOAP remote API. Because of the limitations
 * of the JIRA API (there is no <code>getAllIssues()</code> or
 * <code>getAllIssuesByProject()<code>, and no <code>getProjectById()</code>,
 * it downloads all issues from a saved filter
 * defined by its name, downloads all projects and then sorts
 * the issues for projects.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class JiraSoapServiceImpl extends AbstractJiraServiceImpl implements JiraService {
    private final static Log log = LogFactory.getLog(JiraSoapServiceImpl.class);

    /** SOAP service initialized when connecting. */
    JiraSoapService jiraSoapService = null;

    @SuppressWarnings("unchecked")
    public List<JiraProject> getProjects(String url) {
        log.debug("Returning projects...");

        List<JiraProject> projects = new ArrayList<JiraProject>();
        updateTimestamp();

        beginStoreSession();

        try {
            String token = connect(url);

            // search for the filter
            String filterName = UrlUtils.parseAdditionalItemFromUrl(url);
            RemoteFilter savedFilters[] = jiraSoapService.getSavedFilters(token);
            log.debug(String.format("Searching for \"%s\" in %d saved filters...", filterName, savedFilters.length));
            RemoteFilter remoteFilter = null;
            for(RemoteFilter savedFilter : savedFilters) {
                log.debug(String.format("Checking saved filter \"%s\"...", savedFilter.getName()));
                if(savedFilter.getName().equals(filterName)) {
                    remoteFilter = savedFilter;
                    break;
                }
            }
            if(remoteFilter == null)
                throw new IllegalArgumentException(String.format("Couldn't find saved filter \"%s\"", filterName));

            // download projects
            RemoteProject remoteProjects[] = jiraSoapService.getProjects(token);
            log.debug(String.format("%d remote JIRA projects found", remoteProjects.length));

            // download and analyze issues
            RemoteIssue issues[] = jiraSoapService.getIssuesFromFilter(token, remoteFilter.getId());
            System.out.println(String.format("%d issues downloaded", issues.length));

            for(RemoteProject remoteProject : remoteProjects) {
                log.debug(String.format("Analyzing project \"%s\"...", remoteProject.getName()));

                int openIssues = 0;
                int codingInProgressIssues = 0;
                int reopenedIssues = 0;
                int resolvedIssues = 0;
                int closedIssues = 0;

                int blockerOpenIssues = 0;
                int criticalOpenIssues = 0;
                int majorOpenIssues = 0;
                int minorOpenIssues = 0;
                int trivialOpenIssues = 0;
                int optionalOpenIssues = 0;

                Map<String, Integer> openIssuesPerAssigneeMap = new HashMap<String, Integer>();

                boolean issueFoundForRemoteProject = false;
                for(RemoteIssue issue : issues) {
                    // skip this issue if it doesn't belong to this project
                    if(!remoteProject.getKey().equals(issue.getProject()))
                        continue;
                    issueFoundForRemoteProject = true;

                    // process status
                    int status = Integer.parseInt(issue.getStatus());
                    switch(status) {// TODO move to abs
                        case 1: openIssues++; break;
                        // TODO what is 2?
                        case 3: codingInProgressIssues++; break;
                        case 4: reopenedIssues++; break;
                        case 5: resolvedIssues++; break;
                        case 6: closedIssues++; break;
                        default: throw new IllegalArgumentException(String.format("Unknown issue status: %d", status));
                    }

                    // process priority if open
                    if(status == 1) {// TODO move to abs
                        int priority = Integer.parseInt(issue.getPriority());

                        switch(priority) {
                            case 1: blockerOpenIssues++; break;
                            case 2: criticalOpenIssues++; break;
                            case 3: majorOpenIssues++; break;
                            case 4: minorOpenIssues++; break;
                            case 5: trivialOpenIssues++; break;
                            // TODO what is 6?
                            case 7: optionalOpenIssues++; break;
                            default: throw new IllegalArgumentException(String.format("Unknown issue priority: %d", priority));
                        }
                    }

                    // process assignee
                    ChartUtils.incrementCountMap(openIssuesPerAssigneeMap, issue.getAssignee());
                }

                // skip this project if there is no issues for this
                if(!issueFoundForRemoteProject)
                    continue;

                // generate charts
                String prefix = UrlUtils.removeUserInfoFromUrl(url) + remoteProject.getKey();
                List<Map.Entry<String, Integer>> openIssuesPerAssignee = (List<Map.Entry<String, Integer>>)ChartUtils.countMapToSortedList(openIssuesPerAssigneeMap);

                ZoomableImageLocator issuesPerStatusChartLocator = generateIssuePerStatusChart(prefix, openIssues, codingInProgressIssues, reopenedIssues, resolvedIssues, closedIssues);
                ZoomableImageLocator openIssuesPerPriorityChartLocator = generateIssuePerPriorityChart(prefix, blockerOpenIssues, criticalOpenIssues, majorOpenIssues, minorOpenIssues, trivialOpenIssues, optionalOpenIssues);
                ZoomableImageLocator openIssuesPerAssigneeChartLocator = generateOpenIssuesPerAssigneeChart(prefix, openIssuesPerAssignee);

                // save
                JiraProject project = new JiraProject(remoteProject.getName(), remoteProject.getUrl(), remoteProject.getKey(), remoteProject.getProjectUrl(), remoteProject.getDescription(), remoteProject.getLead(),
                    openIssues, codingInProgressIssues, reopenedIssues, resolvedIssues, closedIssues,
                    blockerOpenIssues, criticalOpenIssues, majorOpenIssues, minorOpenIssues, trivialOpenIssues, optionalOpenIssues,
                    (List<Map.Entry<String, Integer>>)trimList(openIssuesPerAssignee),
                    issuesPerStatusChartLocator, openIssuesPerPriorityChartLocator, openIssuesPerAssigneeChartLocator);
                log.debug(String.format("Adding %s...", project));
                projects.add(project);
            }

            disconnect(token);
        } catch(Exception ex) {
            System.out.println(ex);
        }

        endStoreSession();

        return projects;
    }

    /**
     * Connects to the JIRA SOAP service.
     * @return the session token after a successful connection.
     */
    protected String connect(String url) throws MalformedURLException, ServiceException, RemoteException {
        JiraSoapServiceService jiraSoapServiceGetter = new JiraSoapServiceServiceLocator();
        jiraSoapService = jiraSoapServiceGetter.getJirasoapserviceV2(new URL(url));

        // authenticate if project is secured
        String userInfo[] = UrlUtils.parseUserInfoFromUrl(url);
        if(userInfo != null) {
            String userName = userInfo[0];
            String password = userInfo[1];

            log.debug(String.format("Signing in as \"%s\" \"%s\"...", userName, password));
            String token = jiraSoapService.login(userName, password);
            if(StringUtils.isBlank(token))
                throw new IllegalStateException("Session token is empty, authentication failed");
            log.debug(String.format("Session token: \"%s\"", token));

            return token;
        }

        throw new IllegalArgumentException("URL must contain username and password");
    }

    /** Disconnects from the given JIRA SOAP service. */
    protected void disconnect(String token) throws RemoteException {
        if(!jiraSoapService.logout(token))
            log.error("Couldn't properly logout from remote JIRA session");
    }
}
